#include "mpconfig.h"
#include "misc.h"
#include "nlr.h"
#include "qstr.h"
#include "obj.h"
#include "runtime.h"
#include "objtuple.h"

#include <fxcg/keyboard.h>
#include <fxcg/rtc.h>
int ck_getkey(int * keyptr); // khicas main.cc

static mp_obj_t nsp_readRTC()
{
  return mp_obj_new_int((RTC_GetTicks()*1000)/128);
}
static MP_DEFINE_CONST_FUN_OBJ_0(nsp_readRTC_obj, nsp_readRTC);

static mp_obj_t nsp_waitKeypress()
{
  int key;
  GetKey(&key);
  return mp_const_none;
}
static MP_DEFINE_CONST_FUN_OBJ_0(nsp_waitKeypress_obj, nsp_waitKeypress);

static mp_obj_t nsp_getkey(size_t n_args, const mp_obj_t *args){
  if (n_args==0){
    int key;
    GetKey(&key);
    if (key==KEY_CTRL_LEFT)
      key=0;
    else if (key==KEY_CTRL_UP)
      key=1;
    else if (key==KEY_CTRL_DOWN)
      key=2;
    else if (key==KEY_CTRL_RIGHT)
      key=3;
    else if (key==KEY_CTRL_EXE)
      key=4;
    else if (key==KEY_CTRL_EXIT)
      key=5;
    return mp_obj_new_int(key);
  }
  if (!MP_OBJ_IS_SMALL_INT(args[0]))
    mp_raise_TypeError("getkey() or getkey(-1) or getkey(keycode)");
  int i=MP_OBJ_SMALL_INT_VALUE(args[0]);
  if (i==-1){
    int key=PRGM_GetKey();
    return mp_obj_new_int(key);
  }
  int key=PRGM_GetKey();
  return mp_obj_new_int(key==i);  
}
static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(nsp_getkey_obj, 0,1,nsp_getkey);

static mp_obj_t nsp_sleep(size_t n_args, const mp_obj_t *args){
  if (!MP_OBJ_IS_SMALL_INT(args[0]))
    mp_raise_TypeError("sleep expect an integer (milliseconds)");
  OS_InnerWait_ms(MP_OBJ_SMALL_INT_VALUE(args[0]));
  return mp_const_none;
}
static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(nsp_sleep_obj, 1,1,nsp_sleep);

STATIC const mp_map_elem_t mp_module_nsp_globals_table[] = {
        { MP_ROM_QSTR(MP_QSTR___name__), MP_ROM_QSTR(MP_QSTR_nsp) },
	//{ MP_OBJ_NEW_QSTR(MP_QSTR_Texture), (mp_obj_t) &nsp_texture_type },
	{ MP_OBJ_NEW_QSTR(MP_QSTR_waitKeypress), (mp_obj_t) &nsp_waitKeypress_obj },
	{ MP_OBJ_NEW_QSTR(MP_QSTR_getkey), (mp_obj_t) &nsp_getkey_obj },
	{ MP_OBJ_NEW_QSTR(MP_QSTR_sleep), (mp_obj_t) &nsp_sleep_obj },
	{ MP_OBJ_NEW_QSTR(MP_QSTR_readRTC), (mp_obj_t) &nsp_readRTC_obj },
	{ MP_OBJ_NEW_QSTR(MP_QSTR_monotonic), (mp_obj_t) &nsp_readRTC_obj },
	{ MP_OBJ_NEW_QSTR(MP_QSTR_time), (mp_obj_t) &nsp_readRTC_obj },
};

STATIC const mp_obj_dict_t mp_module_nsp_globals = {
    .base = {&mp_type_dict},
    .map = {
        .all_keys_are_qstrs = 1,
        .is_fixed = 1,
        .used = MP_ARRAY_SIZE(mp_module_nsp_globals_table),
        .alloc = MP_ARRAY_SIZE(mp_module_nsp_globals_table),
        .table = (mp_map_elem_t*) mp_module_nsp_globals_table,
    },
};

const mp_obj_module_t mp_module_nsp = {
    .base = { &mp_type_module },
    .globals = (mp_obj_dict_t*) &mp_module_nsp_globals,
};
